{ ------------------------------------------------------------- }
{                                                               }
{   Name:           XBackCls.pas                                }
{   Version:        V 0.1                                       }
{   Author:         Erick I. Jimenez Alvarado                   }
{   To contact:     erickj@usa.net                              }
{                   http://members.xoom.com/erickj              }
{                                                               }
{   Description:    Class ObjBackground Class                   }
{                                                               }
{   File Status:    COMPONETWARE ( like a FREEWARE but if you   }
{                   make a Componet or modify this please       }
{                   send or e-mail me this).                    }
{                                                               }
{   Last Upadated:  02/01/99                                    }
{                                                               }
{   History:                                                    }
{                                                               }
{    02/01/99   V0.1 -  First relase version. Cool.             }
{                                                               }
{   Finally:        Sorry my bad, bad english and thanks for    }
{                   use my component.                           }
{                                                               }
{ ------------------------------------------------------------- }

{.$DEFINE PRG_UNIT}

{$IFDEF PRG_UNIT}               { ---------------------- }

{$DEFINE PRG_INTERFACE}
{$DEFINE PRG_IMPLEMENTATION}

{$I XTools.Inc } { Directives for all controls }

unit XBackCls;

interface

uses

 {$IFNDEF WIN32}
    WinTypes, WinProcs,             { Default Units }
 {$ELSE}
    Windows,                        { Default Unit  }
 {$ENDIF}  { // WIN32 }

    Graphics, Classes,

    GraphExt,                          { My Unit       }

    XBmpCls, XGradCls;

{$ENDIF} { // PRG_UNIT }


{$IFDEF PRG_INTERFACE}          { ---------------------- }

type

  TObjBackgroundStyle = (efbNone, efbGradient, efbTexture, efbBrush);

{
// --------------------------------------------------------
//  ObjBackGorund Class
// --------------------------------------------------------
}
  TObjBackground = class(TPersistent)
  private

    FLock       : Byte;                 { Lock events                   }

    FStyle      : TObjBackgroundStyle;
    FGradient   : TObjGradient;
    FTexture    : TObjBitmap;
    FBrush      : TBrush;

    FOnChange   : TNotifyEvent;         { handle for change on property }

    procedure   SetGradient(Value : TObjGradient);
    procedure   SetTexture(Value : TObjBitmap);
    procedure   SetBrush(Value : TBrush);
    procedure   SetBackgroundStyle(Value : TObjBackgroundStyle);

    function    GetPalette : HPalette;

  protected

    procedure   Lock;
    procedure   UnLock(Value:Boolean);

  public

    constructor Create; dynamic;
    destructor  Destroy; override;

    procedure   Assign(Source:TPersistent); override;

    procedure   PaintDC(FHandle:HDC; aRect: TRect);

    function    IsEmpty: Boolean;
    function    IsTransparent: Boolean;

    function    PutPalette(DC : HDC) : HPalette; // virtual;

    procedure   GetChange(Sender:TObject); // dynamic;
    procedure   Invalidate; // dynamic;

    property    Palette   : HPalette read GetPalette;

    property    OnChange  : TNotifyEvent read FOnChange write FOnChange;

  published

    property    BackStyle : TObjBackgroundStyle read FStyle write SetBackgroundStyle
                            default efbNone;

    property    Gradient  : TObjGradient read FGradient write SetGradient;
    property    Texture   : TObjBitmap   read FTexture  write SetTexture;
    property    Brush     : TBrush       read FBrush    write SetBrush;

  end;


{$ENDIF} { // PRG_INTERFACE }


{$IFDEF PRG_UNIT}               { ---------------------- }


{
// --------------------------------------------------------
//
// --------------------------------------------------------
}
Implementation

Uses

    SysUtils;                       { Default Units }

{$ENDIF} { // PRG_UNIT }

{$IFDEF PRG_IMPLEMENTATION}     { ---------------------- }

{
// --------------------------------------------------------
//  Default Constructor
// --------------------------------------------------------
}
constructor TObjBackground.Create;
begin

    inherited Create;                   { Parent Create         }

    FLock    := 0;                      { Unlock Refresh and draw   }

    FStyle   := efbNone;

    FGradient := TObjGradient.Create;
    FBrush    := TBrush.Create;
    FTexture  := TObjBitmap.Create;

    FGradient.OnChange := GetChange;
    FBrush.OnChange    := GetChange;
    FTexture.OnChange  := GetChange;

end;

{
// --------------------------------------------------------
//  Default Destructor
// --------------------------------------------------------
}
destructor TObjBackground.Destroy;
begin

    FGradient.OnChange := nil;
    FBrush.OnChange    := nil;
    FTexture.OnChange  := nil;

    FTexture.Free;
    FBrush.Free;
    FGradient.Free;

    inherited Destroy;
end;


{
// --------------------------------------------------------
//  Set Style of Background
// --------------------------------------------------------
}
procedure TObjBackground.SetBackgroundStyle(Value: TObjBackgroundStyle);
begin
    if (Value <> FStyle) then
    begin
        FStyle := Value;
        Invalidate;
    end;
end;

{
// --------------------------------------------------------
//  Copy Gradient Data
// --------------------------------------------------------
}
procedure TObjBackground.SetGradient(Value : TObjGradient);
Begin
    FGradient.Assign(Value);
End;

{
// --------------------------------------------------------
//  Copy Texture Data
// --------------------------------------------------------
}
procedure TObjBackground.SetTexture(Value : TObjBitmap);
Begin
    FTexture.Assign(Value);
End;

{
// --------------------------------------------------------
//  Copy Brush Data
// --------------------------------------------------------
}
procedure TObjBackground.SetBrush(Value : TBrush);
Begin
    FBrush.Assign(Value);
End;

{
// --------------------------------------------------------
//  Copy all data properties to self
// --------------------------------------------------------
}
procedure TObjBackground.Assign(Source:TPersistent);
begin

    if (Source = nil) then
    Begin
      {$IFDEF DEBUGCODE}
        raise Exception.Create('TObjBackground::Assign: Invalid Pointer');
      {$ENDIF} { // DEBUGCODE }
        Exit;
    End;

    if (Source = self) then
        exit;

    { Check if correct class }
    if (Source is TObjBackground) then
    begin

        FLock := 1;                   { Lock Events           }

      try

        FStyle      := TObjBackground(Source).FStyle;

        FBrush.Assign(TObjBackground(Source).FBrush);
        FTexture.Assign(TObjBackground(Source).FTexture);
        FGradient.Assign(TObjBackground(Source).FGradient);

      finally

        FLock := 0;                     { UnLock Events         }
        Invalidate;                     { Notify change             }

      end;

        Exit;

    end;

    { Call inherited }
    inherited Assign(Source);
end;


{
// --------------------------------------------------------
//  Default Self PutPalette
// --------------------------------------------------------
}
function  TObjBackground.PutPalette(DC : HDC) : HPalette;
begin
    if (FStyle = efbTexture) then
        Result := FTexture.PutPalette(DC)
    else if (FStyle = efbGradient) then
        Result := FGradient.PutPalette(DC)
    else Result := 0;
end;


{
// --------------------------------------------------------
//  Default Self PutPalette
// --------------------------------------------------------
}
function  TObjBackground.GetPalette : HPalette;
begin
    if (FStyle = efbTexture) then
        Result := FTexture.Bitmap.Palette
    else if (FStyle = efbGradient) then
        Result := FGradient.Palette
    else Result := 0;
end;



{
// --------------------------------------------------------
// Return if is Transparent
// --------------------------------------------------------
}
function TObjBackground.IsTransparent: Boolean;
begin

    Result := False;

    if (FStyle = efbNone) then
        Result := True;

    with (FTexture) do
    begin
        if (FStyle = efbTexture) and (not (FillInside) or Transparent) then
            Result := True;
    end;
end;

{
// --------------------------------------------------------
// Return if is Empty
// --------------------------------------------------------
}
function TObjBackground.IsEmpty: Boolean;
begin

    Result := False;

    if (FStyle = efbTexture) then
        Result := FTexture.IsEmpty
    else if (FStyle = efbNone) then
        Result := True;
end;

{
// --------------------------------------------------------
//  It correct Repaint Control
// --------------------------------------------------------
}
procedure TObjBackground.PaintDC(FHandle : HDC; aRect: TRect);
Begin

    if (FStyle = efbGradient) then
        FGradient.PaintDC(FHandle,aRect)
    else if (FStyle = efbTexture) then
        FTexture.PaintDC(FHandle,aRect)
    else if (FStyle = efbBrush) then
    begin
        if (FBrush.Style = bsClear) then
            FillRectEx(HDC(Pointer(FHandle)), aRect,
                GetBkColor(HDC(Pointer(FHandle))))
        else
            FillRect(HDC(Pointer(FHandle)),aRect, FBrush.Handle);
    end;
end;

{
// --------------------------------------------------------
//  Get Change event
// --------------------------------------------------------
}
procedure TObjBackground.GetChange(Sender:TObject);
Begin
    Invalidate;
End;


{
// --------------------------------------------------------
//  This procedure is assigned when component changed
// --------------------------------------------------------
}
procedure TObjBackground.Invalidate;
begin

    { Check if lock procedure }
    if (FLock = 1) then
        Exit;

    { Call event on object change }
    if Assigned(FOnChange) then
        FOnChange(Self);

end;

{
// --------------------------------------------------------
//  Lock Events
// --------------------------------------------------------
}
procedure TObjBackground.Lock;
begin
    FLock := 1;
end;

{
// --------------------------------------------------------
//  UnLock Events
// --------------------------------------------------------
}
procedure TObjBackground.UnLock(Value:Boolean);
begin

    FLock := 0;

    if (Value) then
        Invalidate;                         { Notify change             }
end;


{$ENDIF} { // PRG_IMPLEMENTATION }

{$IFDEF PRG_UNIT}               { ---------------------- }
end.
{$ENDIF} { // PRG_UNIT }
