{ ------------------------------------------------------------- }
{                                                               }
{   Name:           XBmpCls.pas                                 }
{   Version:        V 0.2                                       }
{   Author:         Erick I. Jimenez Alvarado                   }
{   To contact:     erickj@usa.net                              }
{                   http://members.xoom.com/erickj              }
{                                                               }
{   Description:    Class to handle EnhBitmap Class             }
{                                                               }
{   File Status:    COMPONETWARE ( like a FREEWARE but if you   }
{                   make a Componet or modify this please       }
{                   send or e-mail me this).                    }
{                                                               }
{   Last Upadated:  02/05/99                                    }
{                                                               }
{   History:                                                    }
{                                                               }
{    01/27/99   V0.1 -  First relase version.                   }
{    02/05/99   V0.2 -  Add Align property                      }
{                                                               }
{   Finally:        Sorry my bad, bad english and thanks for    }
{                   use my component.                           }
{                                                               }
{ ------------------------------------------------------------- }

{.$DEFINE PRG_UNIT}

{$IFDEF PRG_UNIT}               { ---------------------- }

{$DEFINE PRG_INTERFACE}
{$DEFINE PRG_IMPLEMENTATION}

{$I XTools.Inc } { Directives for all controls }

Unit XBmpCls;

interface

uses

 {$IFNDEF WIN32}
    WinTypes, WinProcs,             { Default Units }
 {$ELSE}
    Windows,                        { Default Unit  }
 {$ENDIF}  { // WIN32 }

  Classes, Graphics,                { Default Units }

  GraphExt;                         { My Unit       }

{$ENDIF} { // PRG_UNIT }


{$IFDEF PRG_INTERFACE}          { ---------------------- }


type

  TBitmapDrawStyle = (bmNone, bmAlign, bmStretch, bmTile, bmSmoothResize);

  TTransparentStyle = (tsColor, tsParentColor, tsBitmapColor);

{
// --------------------------------------------------------
//  Bitmap Class
// --------------------------------------------------------
}
  TObjBitmap = Class(TPersistent)

  private

    FLock       : Byte;                 { Lock events                   }
    FTransparent: Boolean;
    FFillInside : Boolean;
    FTransparentStyle: TTransparentStyle;
    FDrawStyle  : TBitmapDrawStyle;
    FAlign      : TEnhAlign;

    FColor      : TColor;
    FBitmap     : TBitmap;

    FOnChange   : TNotifyEvent;         { handle for change on property }


    procedure   SetBitmap(Value: TBitmap);
    procedure   SetDrawStyle(Value: TBitmapDrawStyle);
    procedure   SetColor(Value: TColor);
    procedure   SetAlign(Value: TEnhAlign);

    function    GetIPalette: Boolean;
    procedure   SetIPalette(Value: Boolean);

    function    GetTMode: TTransparentMode;
    procedure   SetTMode(Value: TTransparentMode);
    procedure   SetTStyle(Value: TTransparentStyle);

    procedure   SetTransparent(Value :  Boolean);
    procedure   SetFillInside(Value :  Boolean);

//    function    GetBackColor: TColor;
//    procedure   SetBackColor(Value: TColor);

  protected

    procedure   Lock;
    procedure   UnLock(Value:Boolean);

  public

    constructor Create; dynamic;
    destructor  Destroy; override;


    function    IsEmpty : Boolean;

    procedure   PaintDC(FHandle:HDC; aRect: TRect);       // dynamic;

    procedure   Assign(Source:TPersistent); override;

    procedure   GetChange(Sender:TObject); // override;

    procedure   Invalidate; // dynamic;

  {$IFDEF LIKEPALETTE }
//    function    GetPalette : HPalette;
    function    PutPalette(DC : HDC) : HPalette;
  {$ENDIF}  { // LIKEPALETTE }


    property    OnChange   : TNotifyEvent read FOnChange write FOnChange;

  published

    property    Align      : TEnhAlign read FAlign write SetAlign;

    property    Bitmap     : TBitmap  read FBitmap write SetBitmap;

    property    DrawStyle  : TBitmapDrawStyle read FDrawStyle  write SetDrawStyle
                              default bmNone;

    property    IgnorePalette : Boolean read GetIPalette write SetIPalette
                                default False;

    property    FillInside : Boolean read FFillInside write SetFillInside
                                default False;

    property    Transparent: Boolean read FTransparent write SetTransparent
                                default False;

    property    TransparentStyle : TTransparentStyle read FTransparentStyle write SetTStyle
                                default tsBitmapColor;

    property    TransparentMode  : TTransparentMode  read GetTMode write SetTMode
                                default tmAuto;

    property    Color      : TColor read  FColor write SetColor default clWhite;

  end;


{$ENDIF} { // PRG_INTERFACE }


{$IFDEF PRG_UNIT}               { ---------------------- }

{
// --------------------------------------------------------
//
// --------------------------------------------------------
}
Implementation

{$ENDIF} { // PRG_UNIT }


{$IFDEF PRG_IMPLEMENTATION}     { ---------------------- }


{
// --------------------------------------------------------
//  Default Constructor
// --------------------------------------------------------
}
constructor TObjBitmap.Create;
begin

    inherited Create;                   { Parent Create     }

    FLock    := 0;                      { Unlock Refresh and draw   }

    FBitmap := TBitmap.Create;
    FBitmap.IgnorePalette := False;
    FBitmap.OnChange := GetChange;

    FTransparentStyle := tsBitmapColor;
    FTransparent := False;
    FFillInside  := False;
    FColor       := clWhite;
    FDrawStyle   := bmNone;             { Not Efects }
end;

{
// --------------------------------------------------------
//  Default Destructor
// --------------------------------------------------------
}
destructor TObjBitmap.Destroy;
begin
    FBitmap.OnChange := nil;
    FBitmap.Free;
    inherited Destroy;
end;


{
// --------------------------------------------------------
//  Set Draw Style
// --------------------------------------------------------
}
procedure TObjBitmap.SetDrawStyle(Value: TBitmapDrawStyle);
begin
    if (Value <> FDrawStyle) then
    begin
        FDrawStyle := Value;
        Invalidate;
    end;
end;

{
// --------------------------------------------------------
//  Set Bitmap
// --------------------------------------------------------
}
procedure TObjBitmap.SetBitmap(Value: TBitmap);
begin
//    if (Value.Handle <> FBitmap.Handle) then
//    begin
        FBitmap.Assign(Value);
        // Invalidate; by TBitmap
//    end;
end;

{
// --------------------------------------------------------
//  Set Align
// --------------------------------------------------------
}
procedure TObjBitmap.SetAlign(Value: TEnhAlign);
begin
    if (Value <> FAlign) then
    begin
        FAlign := Value;
        Invalidate;
    end;
end;

{
// --------------------------------------------------------
//  Set Color
// --------------------------------------------------------
}
procedure TObjBitmap.SetColor(Value: TColor);
begin
    if (Value <> FColor) then
    begin
        FColor := Value;
        Invalidate;
    end;
end;

{
// --------------------------------------------------------
//  Get Ignore Palette
// --------------------------------------------------------
}
function TObjBitmap.GetIPalette: Boolean;
begin
    Result := FBitmap.IgnorePalette;
end;

{
// --------------------------------------------------------
//  Set Ignore Palette
// --------------------------------------------------------
}
procedure TObjBitmap.SetIPalette(Value: Boolean);
begin
    if (Value <> FBitmap.IgnorePalette) then
    begin
        FBitmap.IgnorePalette := Value;
        Invalidate; // TBitmap not repaint
    end;
end;

{
// --------------------------------------------------------
//  Get Transparent Mode
// --------------------------------------------------------
}
function TObjBitmap.GetTMode: TTransparentMode;
begin
    Result := FBitmap.TransparentMode;
end;

{
// --------------------------------------------------------
//  Set Transparent Mode
// --------------------------------------------------------
}
procedure TObjBitmap.SetTMode(Value: TTransparentMode);
begin
    if (Value <> FBitmap.TransparentMode) then
    begin
        FBitmap.TransparentMode:= Value;
        // Invalidate; by TBitmap
    end;
end;

{
// --------------------------------------------------------
//  Set Transparent Style
// --------------------------------------------------------
}
procedure TObjBitmap.SetTStyle(Value: TTransparentStyle);
begin
    if (Value <> FTransparentStyle) then
    begin
        FTransparentStyle := Value;
        Invalidate;
    end;
end;

{
// --------------------------------------------------------
//  Set Transparent
// --------------------------------------------------------
}
procedure TObjBitmap.SetTransparent(Value : Boolean);
Begin

    if (Value <> FTransparent) Then
    Begin
        FTransparent := Value;
        Invalidate;
    End;
End;

{
// --------------------------------------------------------
//  Set Fill Inside
// --------------------------------------------------------
}
procedure TObjBitmap.SetFillInside(Value : Boolean);
Begin

    if (Value <> FFillInside) Then
    Begin
        FFillInside := Value;
        Invalidate;
    End;
End;

{
// --------------------------------------------------------
//  Copy all Bitmap properties to self
// --------------------------------------------------------
}
procedure TObjBitmap.Assign(Source : TPersistent);
begin

    if (Source = self) then
        exit;

    if (Source = nil) then
    Begin
      {$IFDEF DEBUGCODE}
        raise Exception.Create('TObjBitmap::Assign: Invalid Pointer');
      {$ENDIF} { // DEBUGCODE }
        exit;
    End;


    { Check if correct class }
    if (Source is TObjBitmap) then
    begin

        FLock := 1;                   { Lock Events           }

      try

        FFillInside := TObjBitmap(Source).FFillInside;
        FTransparent:= TObjBitmap(Source).FTransparent;
        FColor      := TObjBitmap(Source).FColor;
        FAlign      := TObjBitmap(Source).FAlign;
        FDrawStyle  := TObjBitmap(Source).FDrawStyle;
        FTransparentStyle  := TObjBitmap(Source).FTransparentStyle;

        // By TBitmap
        //TransparentMode:= TObjBitmap(Source).TransparentMode;
        IgnorePalette  := TObjBitmap(Source).IgnorePalette;

        FBitmap.Assign(TObjBitmap(Source).FBitmap);

      finally

        FLock := 0;                     { UnLock Events         }
        Invalidate;                     { Notify change             }

      end;

        Exit;

    end;

    { Call inherited }
    inherited Assign(Source);
end;

{
// --------------------------------------------------------
//  Lock Events
// --------------------------------------------------------
}
procedure TObjBitmap.Lock;
begin
    FLock := 1;
end;

{
// --------------------------------------------------------
//  UnLock Events
// --------------------------------------------------------
}
procedure TObjBitmap.UnLock(Value:Boolean);
begin

    FLock := 0;

    if (Value) then
        Invalidate;                         { Notify change             }
end;


{
// --------------------------------------------------------
//  Refresh Data and call Event OnChange
// --------------------------------------------------------
}
procedure TObjBitmap.Invalidate;
begin
    { Check if lock procedure }
    if (FLock = 1) then
        Exit;

    { Correct Align }
    if (FDrawStyle <> bmAlign) then
        FAlign := eaTopLeft;

    if (FTransparent) then
        FFillInside := False;

    if not ((FDrawStyle = bmAlign) or (FDrawStyle = bmNone)) then
        FFillInside := False;

//    if not ((FDrawStyle = bmStrech) or (FDrawStyle = bmSmoothResize)) then
//        FFillInside := False;


    { Call event on object change }
    if Assigned(FOnChange) then
        FOnChange(Self);
end;


{
// --------------------------------------------------------
//  This procedure is assigned when bitmap changed
// --------------------------------------------------------
}
procedure TObjBitmap.GetChange(Sender:TObject);
begin
    Invalidate;
end;

{
// --------------------------------------------------------
// Draw Bitmap Object
// --------------------------------------------------------
}
procedure TObjBitmap.PaintDC(FHandle : HDC; aRect: TRect);

Var
    W,H     : Integer;

    OldBmp  : HBitmap;
    TmpDC,
    BmpDC   : HDC;

    TmpStyle: TBitmapDrawStyle;
    TmpRect : TRect;
    TmpColor: TColorRef;


begin

  {$IFNDEF DEBUGCODE}

    if (FLock = 1) or (FHandle = 0) or IsRectEmpty(aRect)
       or FBitmap.Empty then
        Exit;

  {$ELSE}

    if (FLock = 1) or IsRectEmpty(aRect) or FBitmap.Empty then
        Exit;


    if (FHandle = 0)  then
    Begin
        raise Exception.Create('TObjBitmap::PaintDC: Handle not valid');
        Exit;
    End;

  {$ENDIF} { // DEBUGCODE }

    if (FTransparentStyle = tsColor) then
        TmpColor := ColorToRGB(FColor)
    else if (FTransparentStyle = tsBitmapColor) then
        TmpColor := ColorToRGB(FBitmap.TransparentColor)
    else // if (FTransparentStyle = tsParentColor) then
        TmpColor := GetBkColor(HDC(Pointer(FHandle)));

    TmpStyle := FDrawStyle;

    W := FBitmap.Width;
    H := FBitmap.Height;

    BmpDC := FBitmap.Canvas.Handle;

    if (FFillInside) then
        with aRect do
            FillRectEx(HDC(Pointer(FHandle)), aRect, TmpColor);

    if (FTransparent) then
    begin
        with aRect do
        begin

            TmpDC  := CreateCompatibleDC(HDC(Pointer(FHandle)));
            OldBmp := SelectObject(HDC(Pointer(TmpDC)),
                      CreateCompatibleBitmap( HDC(Pointer(FHandle)),
                                     Right, Bottom));

            PutPalette(TmpDC);

            BitBlt(HDC(Pointer(TmpDC))  , 0, 0, Right, Bottom,
                   //HDC(Pointer(FHandle)), Left, Top, SRCCOPY);
                   HDC(Pointer(FHandle)), 0, 0, SRCCOPY);

        end;
    end
    else
    begin
        TmpDC  := FHandle;
        OldBmp := 0;
    end;

    { Adjust Width & Height }
    With aRect do
    begin
        Right := Right  - Left;
        Bottom:= Bottom - Top;
    end;


  try

    with aRect do
    begin
        if (TmpStyle = bmNone) then
            BitBlt(HDC(Pointer(TmpDC)),Left, Top, W,H,
                   HDC(Pointer(BmpDC)),0,0,SRCCOPY)
        else if (TmpStyle = bmAlign) then
        begin
            TmpRect := AlignRect(aRect,Rect(0,0,W,H),FAlign);

            BitBlt(HDC(Pointer(TmpDC)),TmpRect.Left, TmpRect.Top, W,H,
                   HDC(Pointer(BmpDC)),0,0,SRCCOPY);
        end
        else if (tmpStyle = bmTile) then
            BmpTileDraw(TmpDC,Left, Top, Right, Bottom, BmpDC, W, H)
        else if (tmpStyle = bmStretch) then
            BmpStrechDraw(TmpDC,Left,Top, Right, Bottom, BmpDC, W, H);
//        else if (tmpStyle = bmSmoothResize) then
//            BmpSmResizeDraw(FHandle,aRect,BmpDC, W, H);

        if (FTransparent) then
        Begin

            BmpTransparentDraw(FHandle, Left, Top, Right, Bottom,
                               TmpDC, Left, Top, Right, Bottom,
                               FBitmap.Palette, TColorRef(TmpColor));

        End;

    end;

  finally

    if (FTransparent) then
    Begin
        DeleteObject(SelectObject(HDC(Pointer(TmpDC)),OldBmp));
        DeleteDC(HDC(Pointer(TmpDC)));
    End;

  end;
end;


{
// --------------------------------------------------------
//  Return if is Empty
// --------------------------------------------------------
}
function TObjBitmap.IsEmpty: Boolean;
begin
    Result := FBitmap.Empty;
end;


{$IFDEF LIKEPALETTE }
{
// --------------------------------------------------------
// Set Bitmap Palette
// --------------------------------------------------------
}
function TObjBitmap.PutPalette(DC : HDC): HPalette;
begin
    Result := 0;

    if (FBitmap.IgnorePalette) or (FBitmap.Palette = 0) then
        Exit;

    {  Check for low color ressolution   }
    Result := SelectPalette(HDC(Pointer(DC)), FBitmap.Palette, False);
    RealizePalette(HDC(Pointer(DC)));
end;
{$ENDIF} { // LIKEPALETTE }


//function  TObjBitmap.GetBackColor: TColor;
//begin
//    Result := FBitmap.TransparentColor;
//end;


{$ENDIF} { // PRG_IMPLEMENTATION }

{$IFDEF PRG_UNIT}               { ---------------------- }
end.
{$ENDIF} { // PRG_UNIT }
